;;  ------------------------------------------------------------------------
;; |   FILE           :  swap-as-local-function.rkt                         |
;; |   AUTHOR         :  Eugene Wallingford                                 |
;; |   CREATION DATE  :  2024/03/19                                         |
;; |   DESCRIPTION    :  demonstrating local procedures with let            |
;;  ------------------------------------------------------------------------

#lang racket

;; --------------------------------------------------------------------------
;; The invert procedure swaps the members of all its arguments:
;;
;; > (invert '((eugene 10) (jerry 8) (elaine 7) (george 4)))
;; ((10 eugene) (8 jerry) (7 elaine) (4 george))

;; --------------------------------------------------------------------------
;; version 1: swap as a standard helper function

(define invert-v1
  (lambda (list-of-2lists)
    (map swap list-of-2lists)))

(define swap
  (lambda (2-list)
    (list (second 2-list) (first 2-list))))

;; --------------------------------------------------------------------------
;; version 2: swap as a local function

(define invert-v2
  (lambda (list-of-2lists)
    (let ((swap (lambda (lst)
                  (list (second lst) (first lst)) )))
      (map swap list-of-2lists))))

;; --------------------------------------------------------------------------
;; version 3: program derivation

(define invert
  (lambda (list-of-2lists)
    (map (lambda (lst)
           (list (second lst) (first lst)))
         list-of-2lists)))

;; --------------------------------------------------------------------------

